# -*- coding: utf-8 -*-
"""
Created on Mon Jul  1 15:29:46 2013

@author: delande
"""

# Copyright Dominique Delande
# Provided "as is" without any warranty
#
# Computes the diffusion constant (in momentum space)
# for the standard map as a function of the kick strength K
# Initial conditions are generated randomly
# Because of large individual fluctuations among trajectories,
# an accurate estimation requires averaging over a decent number of trajectories
# (of the order of 1000)
# The variance of <p**2(t)> is also computed,
# and an error bar printed in the third column
# The simple analytic expressions for the diffusion constant are also
# printed in the second and third curves.
# One can observe sharp peaks just above 2*n*pi (the most visible ones are for n=1 and n=2)
# These are due to accelerator modes which occupy a small area in phase space,
# but give huge contributions to the diffusion constant, as <p**2> there increases
# like t**2, asymptotically dominating the diffusive contribution 2*D*t.
# The accelator mode also manifests itself by a large increase in the error bar

from math import *
import time
import socket
import os
import random
import numpy
from scipy import special

dpi=2.0*pi
# 100 trajectories for a total execution time of 10 seconds
# 10000 needed for a nice smooth curve (1000 seconds...)
number_of_trajectories=100
final_time=100
K_min=0.0
K_max=40.01
K_step=0.2
diff_file=open('diffusion_constant_vs_K.dat','w')
print >>diff_file,'# Data generated on',time.asctime(),' using Python script',os.path.abspath( __file__ ),' on computer:',socket.gethostname()
print >>diff_file,'# Diffusion constant versus K for the standard map'
print >>diff_file,'# number_of_trajectories =',number_of_trajectories
print >>diff_file,'# final_time =',final_time
for K in numpy.arange(K_min,K_max,K_step):
  print 'K=',K  
  p2_tot=0.0
  p2_tot_2=0.0
  for i_traj in range(number_of_trajectories):
    theta=random.uniform(0,dpi)
    I=random.uniform(0,dpi)-pi
    p2_ini=I*I
    for i in range(final_time):
      I+=K*sin(theta)
      theta+=I
    p2_tot+=(I*I-p2_ini)  
    p2_tot_2+=(I*I-p2_ini)**2
  p2_tot_average=p2_tot/number_of_trajectories
  p2_tot_sigma=sqrt((p2_tot_2/number_of_trajectories-p2_tot_average**2)/number_of_trajectories)
  print >>diff_file,K,p2_tot_average/(2.0*final_time),p2_tot_sigma/(2.0*final_time)
# Flush the output; this makes it possible to stop the calculation and preserve the points already computed  
  diff_file.flush()
print >>diff_file,' '  
# Then plot the simplest analytic expression K*K/4
for K in numpy.arange(K_min,K_max,K_step):
  print >>diff_file,K,0.25*K*K,'0.0'
print >>diff_file,' '  
# Then plot the "analytic" expression using Bessel functions
for K in numpy.arange(K_min,K_max,K_step):
  print >>diff_file,K,0.25*K*K*(1.0-2.0*special.jn(2,K)+2.0*special.jn(2,K)**2),'0.0'
diff_file.close()
