# -*- coding: utf-8 -*-
"""
Created on Mon Jul  1 15:29:46 2013

@author: delande
"""

# Copyright Dominique Delande
# Provided "as is" without any warranty
#
# Computes the distribution of final momenta
# for a set of trajectories of the standard map
# Initial conditions are generated randomly in the [-pi,pi]x[-pi,pi] square
# Prints the histogram of the distribution of final momentum
# If the variable detailed_output is set to True, it also
# prints the individual final positions and momenta
# This makes is possible to visualize diffusion is momentum space
# and complete delocalization in configuration space
# 
# The histogram should be a Gaussian in the fully chaotic regime, i.e. K>7
# Trapping near the origin takes place for K<K_c=0.97
# For K between 1 and 4, part of the trajectories are trapped in stable islands
# leading to narrow peaks in the histogram, very visible at long times
# This is especially true for K between 4 and 2*pi, where there is practically
# a single stable island (of order 2, thus 2 peaks)

from math import *
import time
import socket
import random
import os
import numpy

dpi=2.0*pi
# for generating reproducible random sequences
# comment out for irreproducible sequences  
random.seed(123)  
# The 6 following lines for the fully chaotic regime with a perfect Gaussian distribution
# Execution time: 15 seconds 
K=8.0
number_of_trajectories=100000
final_time=100
bin_size=8.0
detailed_output=False # Too many trajectories to keep all of them! 
number_of_bins=100

# The 6 following lines for the not fully chaotic regime, 
# producing a  Gaussian distribution with a tail
# Execution time: 15 seconds 
#K=7.0
#number_of_trajectories=100000
#final_time=100
#bin_size=8.0
#detailed_output=False # Too many trajectories to keep all of them! 
#number_of_bins=100

# The 6 following lines for the not fully chaotic regime, 
# producing extra an extra peak due to stability islands
# Execution time: 14 seconds 
#K=4.0
#number_of_trajectories=10000
#final_time=1000
#bin_size=6.0
#detailed_output=False # Too many trajectories to keep all of them! 
#number_of_bins=100

I_max=number_of_bins*bin_size/2.0
I_min=-I_max
population_bins=numpy.zeros(number_of_bins)

if (detailed_output):
  output_file=open('final_momentum_distribution.dat','w')
  print >>output_file,'# Data generated on',time.asctime(),' using Python script',os.path.abspath( __file__ ),' on computer:',socket.gethostname()
  print >>output_file,'# Final momentum distribution of the standard map'
  print >>output_file,'# K =',K
  print >>output_file,'# number_of_trajectories =',number_of_trajectories
  print >>output_file,'# final_time =',final_time
for i_traj in range(number_of_trajectories):
  theta=random.uniform(0,dpi)
  I=random.uniform(0,dpi)-pi
  for i in range(final_time):
    I+=K*sin(theta)
    theta+=I
  if (detailed_output): 
    print >>output_file,theta,I
  i_bin=int(floor((I-I_min)/bin_size))
  if ((i_bin>=0)&(i_bin<number_of_bins)):
    population_bins[i_bin]+=1.0   
population_bins/=(bin_size*number_of_trajectories)   
if (detailed_output): 
  output_file.close()
bin_file=open('final_momentum_histogram.dat','w')
print >>bin_file,'# Data generated on',time.asctime(),' using Python script',os.path.abspath( __file__ ),' on computer:',socket.gethostname()
print >>bin_file,'# Histogram of final momenta for the standard map'
print >>bin_file,'# K =',K
print >>bin_file,'# number_of_trajectories =',number_of_trajectories
print >>bin_file,'# final_time =',final_time
print >>bin_file,'# number_of_bins =',number_of_bins
print >>bin_file,'# bin_size =',bin_size
for i_bin in range(number_of_bins):
  print >>bin_file,I_min+i_bin*bin_size,population_bins[i_bin]
  print >>bin_file,I_min+(i_bin+1)*bin_size,population_bins[i_bin]
bin_file.close()