# -*- coding: utf-8 -*-
"""
Created on Mon Jul  1 15:29:46 2013

@author: delande
"""
# Copyright Dominique Delande
# Provided "as is" without any warranty
#
# Computes the average value of <p**2(t)> as a function of time
# for the standard map
# Initial conditions are generated randomly
# For a single initial condition, it displays large fluctuations
# When averaged over several initial conditions, it displays
# an average linear increase in time, characteristic of a 
# (chaotic) diffusion process, for K sufficiently large,
# say K>2
# For K<1, momentum is bounded by insulating invariant tori
# leading to saturation of <p**2(t)> at long time.
# For K slightly above 2*n*pi, accelerator modes
# should produce a quadratic growth; hard to observe in
# practice, except for carefully chosen initial conditions
# 10 initial conditions are enough for characterizing the diffusion process
# 10000 if you want a perfect linea growth
# Execution time scales like number_of_tajectories*final_time
# For 5000 trajectories up to final_time 400, it takes 10 seconds
# on a slow moderately old (2009) laptop

from math import *
import time
import socket
import random
import os
import numpy

dpi=2.0*pi
K=11.6
number_of_trajectories=5000
final_time=400
tab_p2=numpy.zeros(final_time+1)
for i_traj in range(number_of_trajectories):
  theta=random.uniform(0,dpi)
  I=random.uniform(0,dpi)-pi
  tab_p2[0]+=I*I
  for i in range(1,final_time+1):
    I+=K*sin(theta)
    theta+=I
    tab_p2[i]+=I*I   
p2_file=open('p2_vs_time_classical_periodic_kr.dat','w')
print >>p2_file,'# Data generated on',time.asctime(),' using Python script',os.path.abspath( __file__ ),' on computer:',socket.gethostname()
print >>p2_file,'# <p**2> versus time for the standard map'
print >>p2_file,'# K =',K
print >>p2_file,'# number_of_trajectories =',number_of_trajectories
print >>p2_file,'# final_time =',final_time
for i in range(final_time+1):
  print >>p2_file,i,tab_p2[i]/number_of_trajectories
p2_file.close()